from kiper.pipeline.pipeline import PipelineManager

# initialize pipeline
manager = PipelineManager()
manager.load_pipelines()

# Define order for the piplines by foldername
pipeline_order = ["example_pipeline",]

results = manager.run(pipeline_order)
for pipeline_name, result in results.items():
    print(f"Results for pipeline '{pipeline_name}': {result}")



import click
import os
import shutil
import toml
from pathlib import Path

@click.command()
@click.option('--project-name', prompt='Please enter the project name (optional)', default="project", help='Name of the project')
@click.option('--pipeline-name', prompt='Please enter the pipeline name (optional)', default='pipelines', help='Name of the pipeline Folder')
def setup_project(project_name, pipeline_name):
    # Bestimme den Pfad des Projektverzeichnisses
    project_folder = os.path.abspath(project_name)

    # Erstelle das Projektverzeichnis
    os.makedirs(project_folder, exist_ok=True)

    # Erstelle die project.toml-Datei
    project_toml = {
        'project': {
            'name': project_name,
            'place': pipeline_name,
        }
    }

    # Speichere die project.toml-Datei
    with open(os.path.join(project_folder, 'project.toml'), 'w') as f:
        toml.dump(project_toml, f)

    # Erstelle die __init__.py Datei im Projektverzeichnis
    init_file = os.path.join(project_folder, "__init__.py")
    Path(init_file).touch()

    # Bestimme den Pfad des Vorlagenordners (templates folder)
    templates_folder = os.path.join(os.path.dirname(__file__), '..', 'templates')

    # Kopiere die Datei main.py aus dem Vorlagenordner in das Projektverzeichnis
    shutil.copy(os.path.join(templates_folder, 'main.py'), project_folder)

    # Erstelle einen neuen Ordner mit dem Namen pipeline_name im Projektverzeichnis
    pipeline_folder = os.path.join(project_folder, pipeline_name)
    os.makedirs(pipeline_folder, exist_ok=True)

    # create data folder
    shutil.copy(os.path.join(templates_folder, 'data.py'), project_folder)

    # Gehe in den Pipelinenordner und erstelle einen Unterordner "example_pipeline"
    example_pipeline_folder = os.path.join(pipeline_folder, 'example_pipeline')
    os.makedirs(example_pipeline_folder, exist_ok=True)

    # Erstelle die __init__.py Datei im Unterordner "example_pipeline"
    init_file_example = os.path.join(example_pipeline_folder, "__init__.py")
    Path(init_file_example).touch()

    # Kopiere die Dateien pipeline.py und example_node.py in den Unterordner "example_pipeline"
    shutil.copy(os.path.join(templates_folder, 'pipeline.py'), example_pipeline_folder)
    shutil.copy(os.path.join(templates_folder, 'example_node.py'), example_pipeline_folder)

    # Bestimme den Pfad des Quellordners (source folder)
    source_folder = os.path.join(os.path.dirname(__file__), '..', 'source')

    # Kopiere den Quellordner (source folder) in das Projektverzeichnis
    if os.path.exists(source_folder):
        shutil.copytree(source_folder, os.path.join(project_folder, os.path.basename(source_folder)))

    click.echo(f"Project setup complete. Project directory created: {project_folder}")

if __name__ == '__main__':
    setup_project()



from kiper.pipeline.pipeline import Pipeline
from kiper.pipeline.node import Node
from pipelines.example_pipeline.example_node import node_function_1, node_function_2

# Dont change name and directory of __file__ (pipline.py)

def register() -> Pipeline:
    # Define notes for the pipeline
    nodes = [
        Node(name='node1', function=node_function_1, inputs=[1, 2]),
        Node(name='node2', function=node_function_2, inputs=[1, 2])
    ]


    pipeline = Pipeline() 
    for node in nodes:
        pipeline.add_node(node.name, node.function, node.inputs)

    return pipeline


import pandas as pd

class Data:
    def __init__(self, name, data, description=None):
        """
        Initialisiert eine Datenklasse.
        
        :param name: Der Name der Daten.
        :param data: Die Daten selbst.
        :param description: Eine optionale Beschreibung der Daten.
        """
        self.name = name
        self.data = data
        self.description = description

class DataManager:
    def __init__(self, data_classes):
        self.data = {}
        self.auto_add(data_classes)

    def auto_add(self, data_classes):
        for data_class in data_classes:
            instance = data_class()
            self.add(instance.name, instance, instance.description)


    def add(self, name, value, description=None):
        """
        Fügt dem DataManager ein neues Datenobjekt hinzu.
        :param name: Der Name des Datenobjekts.
        :param value: Der Wert des Datenobjekts.
        :param description: Die optionale Beschreibung des Datenobjekts.
        """
        self.data[name] = {'value': value, 'description': description}

    def get(self, name):
        """
        Ruft ein gespeichertes Datenobjekt aus dem DataManager ab.
        :param name: Der Name des abzurufenden Datenobjekts.
        :return: Das abgerufene Datenobjekt.
        """
        return self.data.get(name)

    def update(self, name, new_value):
        """
        Aktualisiert ein gespeichertes Datenobjekt im DataManager.
        :param name: Der Name des zu aktualisierenden Datenobjekts.
        :param new_value: Der neue Wert für das Datenobjekt.
        """
        if name in self.data:
            self.data[name]['value'] = new_value
            print(f"Datenobjekt '{name}' erfolgreich aktualisiert.")
        else:
            print(f"Datenobjekt '{name}' nicht gefunden. Kann nicht aktualisiert werden.")

    def list(self):
        """
        Listet alle gespeicherten Datenobjekte im DataManager auf.
        """
        print("Gespeicherte Datenobjekte:")
        for name, data_info in self.data.items():
            value_type = type(data_info['value'])
            description = data_info['description']
            print(f"{name}: {value_type} - {description}")


# Beispielklassen für Daten
"""
class SapData(Data):
    def __init__(self):
        super().__init__(
            name="SAP-Daten",
            data="asd",
            description="Daten aus dem SAP-System"
        )

class DrcData(Data):
    def __init__(self):
        super().__init__(
            name="DRC-Daten",
            data=None,
            description="Daten aus dem DRC-System"
        )

# Beispielverwendung des DataManager

def main():
    data_manager = DataManager([SapData, DrcData])
    data_manager.list()

if __name__ == "__main__":
    main()
"""


from kiper.data.data import Data

class YourDataClass(Data):
    def __init__(self):
        super().__init__(
            name="Name",
            data="Data",
            description="Description"
        )


import os
import importlib.util
from pathlib import Path
import toml
from .node import Node
import inspect

class Pipeline:
    def __init__(self):
        self.nodes = {}
      

    def run(self):
        outputs = {}
        for node_name, node in self.nodes.items():
            outputs[node_name] = node.execute()
        return outputs

    def add_node(self, name, func, inputs):
        self.nodes[name] = Node(name, func, inputs)

    def register(self, pipeline_instance):
        for pipline in pipeline_instance:
            if pipline not in self.nodes:
                raise ValueError(f"Pipeline '{pipline}' ist erforderlich, wurde aber nicht gefunden.")
        self.nodes.update(pipeline_instance.nodes)


class PipelineManager:
    def __init__(self):
        self.pipelines = {}

    def load_pipelines(self, base_path=None):
        if base_path is None: base_path = Path(os.path.dirname(os.path.abspath(inspect.stack()[1].filename)))

        # Lade project.toml-Datei und extrahiere den Pfad und die Namen der Pipelines
        project_toml_path = base_path / "project.toml"
        with open(project_toml_path, "r") as f:
            project_config = toml.load(f)
            pipeline_folder_name = project_config["project"]["place"]
            pipeline_folder = base_path / pipeline_folder_name
        
            for pipeline_name in pipeline_folder.iterdir():
                if pipeline_name.is_dir():
                    pipeline_name = pipeline_name.name
                    pipeline_module_path = pipeline_folder / pipeline_name / "pipeline.py"
                    if pipeline_module_path.exists():
                        spec = importlib.util.spec_from_file_location("pipeline_module", pipeline_module_path)
                        pipeline_module = importlib.util.module_from_spec(spec)
                        spec.loader.exec_module(pipeline_module)
                        pipeline_register_function = getattr(pipeline_module, "register", None)
                        if callable(pipeline_register_function):
                            pipeline_definition = pipeline_register_function()
                            if isinstance(pipeline_definition, Pipeline):
                                self.pipelines[pipeline_name] = pipeline_definition
                            else:
                                raise ValueError(f"The register function in {pipeline_module_path} must return a Pipeline instance.")

    def run(self, pipeline_order):
        results = {}
        for pipeline_name in pipeline_order:
            if pipeline_name in self.pipelines:
                pipeline = self.pipelines[pipeline_name]
                results[pipeline_name] = pipeline.run()
            else:
                raise ValueError(f"Pipeline '{pipeline_name}' not found.")

        return results


class Node:
    def __init__(self, name, function, inputs):
        self.name = name
        self.function = function
        self.inputs = inputs

    def execute(self):
        return self.function(*self.inputs)